/**
 * Kivu4j. Copyright (C) 2010 . see<http://kivu4j.sourceforge.net />

   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses />.
 */

package net.sourceforge.kivu4j.utils.api.facade;

import java.io.Serializable;
import java.util.Collection;

import net.sourceforge.kivu4j.utils.lang.domain.Dto;
import net.sourceforge.kivu4j.utils.lang.domain.Filter;


/**
 * 泛型接口
 * 
 *
 * @author Lucky Yang
 */
public interface  GenericFacade<D extends Dto<PK>,PK extends Serializable> extends Facade{
	/**
	 * find with the given identifier
	 * @param id identifier
	 * @return data transfer object
	 */
	D findById(PK id);

	/**
	 * find with the given filters
	 * @param filters
	 * @param start page start
	 * @param limit page size
	 * @return collection of dto
	 * @see org.megaminx.core.orm.Filter
	 */
	Collection<D> filter(Collection<Filter> filters, int start, int limit);
	
	/**
	 * find with the given example dto 
	 * @param dto
	 * @param start page start
	 * @param limit page size
	 * @return collection of dto
	 */
	Collection<D> findByExample(D dto,int start,int limit);
	
	/**
	 * create a new dto
	 * 
	 * @return data transfer object
	 */
	D create();
	
	/**
	 * save dto
	 * 
	 * @param dto data transfer object
	 * @return data transfer object
	 */
	D save(D dto);
	
	/**
	 * delete dto
	 * @param dto data transfer object
	 */
	void delete(D dto);
}
