/**
 * Kivu4j. Copyright (C) 2010 . see<http://kivu4j.sourceforge.net />

   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses />.
 */

package net.sourceforge.kivu4j.utils.api.repository;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import net.sourceforge.kivu4j.utils.lang.domain.Filter;

import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;

/**
 * Filter查询参数
 *
 * @author Lucky Yang
 */
public class FilterParameter extends Parameter { 
	public final static String DEFAULT_ALIAS ="DEFAULT_ALIAS";
	private Map<String, List<Filter>> parameters = new HashMap<String, List<Filter>>();
	
	
	public FilterParameter(Collection<Filter> filters) {
		super();
		this.add(filters);
	}

	public FilterParameter(Collection<Filter> filters, int start, int limit) {
		super(start, limit);
		this.add(filters);
	}
	
	public void add(Filter filter){
		String alias = filter.getAlias();
		alias = alias == null? DEFAULT_ALIAS: alias;
		if (this.parameters.containsKey(alias)){
			this.parameters.get(alias).add(filter);
		}else{
			List<Filter> filters = new ArrayList<Filter>();
			filters.add(filter);
			this.parameters.put(alias, filters);
		}
	}
	
	public void add(Collection<Filter> filters){
		for(Filter filter:filters){
			this.add(filter);
		}
	}
	
	public void clear(){
		this.parameters.clear();
	}

	public boolean containAlias(String alias){
		return this.parameters.containsKey(alias);
	}
	
	public List<Filter> getFilters(String key){
		return this.parameters.get(key);
	}
		
	public Map<String, List<Filter>> getParameters() {
		return parameters;
	}

	@Override
	public boolean isValid() {
		return (this.parameters != null);
	}

	@Override
	public String toString() {
		return ToStringBuilder.reflectionToString(this, ToStringStyle.MULTI_LINE_STYLE);
	}
	

}
