/**
 * Kivu4j. Copyright (C) 2010 . see<http://kivu4j.sourceforge.net />

   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses />.
 */

package net.sourceforge.kivu4j.utils.api.repository;

import java.io.Serializable;
import java.util.Collection;

import net.sourceforge.kivu4j.utils.lang.domain.Persistent;

/**
 * 泛型数据访问接口
 * 
 *
 * @author Lucky Yang
 */
public interface GenericRepository<P extends Persistent<PK>, PK extends Serializable> {

	/**
	 * 保存 
	 * 
	 * @param the transient instance to persist 
	 * @return the persistent instance
	 */
	P save(P object);

	/**
	 * 用Id标识查询
	 * 
	 * @param id the identifier of the persistent instance 
	 * @return the persistent instance
	 */
	P findById(PK id);
	
	/**
	 * 用Id标识查询 
	 * 
	 * @param id the identifier of the persistent instance 
	 * @return the persistent instance 
	 */
	P findReferenceById(PK id);
	
	/**
	 * 用Id标识查询
	 * 
	 * @param id the identifier of the persistent instance 
	 * @return the persistent instance 
	 */	
	P findRequiredById(PK id);
	
	/**
	 * 删除
	 * 
	 * @param object the persistent instance to delete 
	 */
	void remove(P object);
	
	/**
	 * 删除
	 * 
	 * @param id the identifier of the persistent instance 
	 */
	void remove(PK id);
	
	/**
	 * 对象是否存在
	 * 
	 * @param id the identifier of the persistent instance 
	 * @return exist return true, or false
	 */
	boolean exist(PK id);

	/**
	 * 使用Example查询参数查询  
	 * 
	 * @param parameter example parameter instance
	 * @return a Collection containing 0 or more persistent instances 
	 */
	Collection<P> find(ExampleParameter parameter);

	/**
	 * 使用Filter查询参数查询  
	 * 
	 * @param parameter filter parameter instance
	 * @return a Collection containing 0 or more persistent instances 
	 */
	Collection<P> find(FilterParameter parameter);

	/**
	 * 使用Query查询参数查询 
	 * 
	 * @param parameter query parameter instance
	 * @return a Collection containing 0 or more persistent instances 
	 */	
	Collection<P> find(QueryParameter parameter);

	/**
	 * 使用Query查询参数查询，返回特定类型的对象集合
	 * 
	 * @param <T>
	 * @param parameter query parameter instance
	 * @param clazz class
	 * @return  a Collection containing 0 or more instances of the clazz
	 */
	<T> Collection<T> find(QueryParameter parameter, Class<T> clazz);

	/**
	 * 使用Query查询参数批量处理 
	 * 
	 * @param parameter query parameter instance
	 * @return the number of instances updated/deleted 
	 */		
	int bulk(QueryParameter parameter);

}